/*
  WeatherC.h - Library for providing data to Weathercloud.net
  Created by: David Snell
  Last Update: 12/4/2015
	
  Created an array of integers "SensorReadings" that can store all
  of Weathercloud's currently accepted settings (as of their 0.6 API). Every SensorReadings
  value is defaulted to -1000, a value Weathercloud will not accept, and that value is
  used to ignore any sensors that are not being used. If you use a sensor please make
  sure to have its value go to the correct array value and in the correct range that is
  appropriate for that sensor (check the Weathercloud API). I do not plan on having any error
  checking of values being sent to Weathercloud.

  There is also an array of strings "SensorNames" that has the name for each variable
  to be posted to Weathercloud.
	
  In order to reduce the memory needed for this (which is pretty high) it is possible to remove
  SensorNames and SensorReadings from their arrays and reduce the MaxSensors value accordingly.
  You must still make sure you are sending the correct names and readings.
*/

#ifndef WeatherC_h
#define WeatherC_h

#include "Arduino.h"
#include "Ethernet.h"

class WeatherC {
  public:
    void Init( );
    void PostData();
    EthernetClient WeatherClient; // Creating a client using the EthernetClient
    char ID[ 17 ]; // The ID value that Weathercloud is expecting.
    char Key[ 33 ]; // The Key value that Weathercloud is expecting.
    const int MaxSensors = 29; // Max number of sensors that can be used.
    int SensorReadings[ 29 ]; // Array to hold all the sensor readings listed in the below table.
    String SensorNames[ 29 ] = { "tempin", "temp", "tempXX", "tempagroXX", "chill", "dewin", "dew", "heatin", "heat", "thw", "humin",
    "hum", "humXX", "wspd", "wspdXX", "wspdhi", "wspdhiXX", "wspdavg", "wspdavgXX", "wdir", "wdirXX", "wdiravg", "wdiravgXX", "bar",
    "rain", "rainrate", "et", "solarrad", "uvi" };
      // Additional names to add as Weathercloud builds support would be: 
      // "soilmoistXX", "leafwetXX", "bartrend", "time", "forecast", "forecasticon", "battery", "batteryXX"
			
      /* Labeled as SensorReadings Array # = SensorNames name = Description
      0 = tempin = Temperature indoors (tempin) in Celsius
      1 = temp = Temperature in Celsius
      2 = tempXX = Temperature in Celsius
      3 = tempagroXX = Temperature in Celsius
      4 = chill = Temperature in Celsius
      5 = dewin = Temperature in Celsius
      6 = dew = Temperature in Celsius
      7 = heatin = Temperature in Celsius
      8 = heat = Temperature in Celsius
      9 = thw = Temperature in Celsius
      10 = humin = Humidity indoors (humin) in %.
      11 = hum = Humidity (hum) in %.
      12 = humXX = Humidity (humXX) in %.
      13 = wspd = Wind speed (wspd) in m/s.
      14 = wspdXX = Wind speed (wspdXX) in m/s.
      15 = wspdhi = Wind speed (wspdhi) in m/s.
      16 = wspdhiXX = Wind speed (wspdhiXX) in m/s.
      17 = wspdavg = Wind speed (wspdavg) in m/s.
      18 = wspdavgXX = Wind speed (wspdavgXX) in m/s.
      19 = wdir = Wind direction (wdir) in degrees.
      20 = wdirXX = Wind direction (wdirXX) in degrees.
      21 = wdiravg = Wind direction (wdiravg) in degrees.
      22 = wdiravgXX = Wind direction (wdiravgXX) in degrees.
      23 = bar = Atmospheric pressure (bar) in hectopascals.
      24 = rain = Daily total rain (rain) in mm.
      25 = rainrate = Rate of rain (rainrate) in mm per hour.
      26 = et = Daily total ET (et) in mm.
      27 = solarrad = Solar radiation (solarrad) in watts per square meter.
      28 = uvi = UV index (uvi).
      // Types below this point are not yet accepted but are in the API
      29 = soilmoistXX = Soil moisture (soilmoist) in centibars (Cb).
      30 = leafwetXX = Leaf wetness (leafwetXX).
      31 = bartrend = (bartrend)
      32 = time = Time in hh:mm [0, 2400].
      33 = forecast = (forecast)
      34 = forecasticon = (forecasticon)
      35 = battery = Battery voltage (battery) in V.
      36 = batteryXX = Battery state (batteryXX) 0 = low, 1 = OK.
      */
			
  private:
    char TargetDNS[ 21 ] = "api.weathercloud.net"; // The server the data is sent to.
    int TargetPort = 80; // Port that the data is sent to.
    double Version = 1.4; // Version information that Weathercloud is expecting.
    int Type = 921; // Type information that Weathercloud is expecting.
    unsigned long PostTime = 300000; // Initial delay of 5 minutes after boot.
    unsigned long PostDelay = 600000; // Delay between data postings of 10 minutes. Weathercloud will not accept faster than 10 min.
};

#endif