/*
  Name: WeatherC Example
  Created by: David Snell
  Last Update: 12/4/2015
  Description: An Arduino Uno based project that uses an Ethernet shield and a variety of sensors
  to take readings and send them to Weathercloud.net. There are a number of variables that
  must be identified for an individual network as well as Weathercloud account. This file
  is only meant to be a sample of what could be done and WILL NOT WORK without at least
  the network settings configured as well as a Weathercloud account, ID, and Key associated.

  This sample project is now using a library I made "WeatherC" to create the sensor reading
  variables to store the data in as well as to regularly post it to Weathercloud.
	
  You will need to obtain an ID and Key from Weathercloud when you set up your account.
*/

// Begin Pin Assignments for Uno
/*
  Pin 0 = RX0 - 
  Pin 1 = TX0 - 
  Pin 2 = DIO - 
  Pin 3 = PWM - 
  Pin 4 = DIO - 
  Pin 5 = PWM - 
  Pin 6 = PWM - Input from DHT-22 or equivalent
  Pin 7 = DIO - 
  Pin 8 = DIO - 
  Pin 9 = PWM - 
  Pin 10 = PWM - Ethernet Shield
  Pin 11 = PWM - Ethernet Shield
  Pin 12 = DIO - Ethernet Shield
  Pin 13 = DIO - Ethernet Shield
  Pin A0 = AI - 
  Pin A1 = AI - 
  Pin A2 = AI - 
  Pin A3 = AI - 
  Pin A4 = AI - 
  Pin A5 = AI - 
*/
// End PIN Assignments

// Begin Includes
#include <SPI.h>
#include <Ethernet.h>
#include <EthernetUdp.h>
#include <DHT.h>
#include <WeatherC.h>
// End Includes

// Begin Network Settings
byte mac[] = { 0x01, 0x01, 0x01, 0x01, 0x01, 0x01 }; // MAC address for ethernet shield goes here, replace the 01s
IPAddress ip( 192, 168, 1, 2 ); // Static IP for ethernet shield. Replace with your IP address.
unsigned int LocalUDP = 8888; // Local UDP port
// End Network Settings

// Begin Weathercloud identifiers
char WeatherID[] = ""; // Enter the ID that Weathercloud provides here. Will not work without it.
char WeatherKey[] = ""; // Enter the Key that Weathercloud provides here. Will not work without it.
// End Weathercloud identifiers

// Begin Sensor Settings
// For my sensors I was using a DHT of one form or another. You could use anything really.
#define DHT_PIN 6 // PIN that the DHT is connected to
#define DHT_TYPE DHT22   // DHT 22 (AM2302) was used in my example
//#define DHT_TYPE DHT11   // DHT 11
unsigned int SensorDelay = 5000; // Set the delay between temperature readings. Default is 5 seconds (5000 milliseconds)
unsigned long SensorTime = 5000; // Set the initial time before a sensor reading occurs
//End Sensor Settings

// Begin Time related
unsigned int UTCTime[ 2 ]; // Array of the segments of time once separated.
unsigned long NTPTime = 1000; // Set the base timing value
unsigned int NTPDelay = 30000; // Delay between readings. Default is 30 seconds (30000 milliseconds)
char timeServer[] = "time.nist.gov"; // time.nist.gov NTP server
const int NTP_PACKET_SIZE = 48; // NTP time stamp is in the first 48 bytes of the message
byte packetBuffer[ NTP_PACKET_SIZE]; //buffer to hold incoming and outgoing packets
// End Time related section

// Begin Initializing Functions
DHT dht( DHT_PIN, DHT_TYPE ); // Initializer for DHT
WeatherC WC; // Initializer for the WeatherC
EthernetUDP Udp; // Ethernet UDP functions
EthernetServer Server( 80 ); // Setting the web server
//End Initializing Functions

// Begin General
bool Debug = false; // Is debugging enabled? Typically false once coding is settled a bit.
bool Networked = true; // Is networking enabled? Typically true but sometimes disabled while coding.
// End General

// Begin sendNTPpacket
unsigned long sendNTPpacket( char* address ){
  // set all bytes in the buffer to 0
  memset( packetBuffer, 0, NTP_PACKET_SIZE );
  // Initialize values needed to form NTP request
  // (see URL above for details on the packets)
  packetBuffer[0] = 0b11100011;   // LI, Version, Mode
  packetBuffer[1] = 0;     // Stratum, or type of clock
  packetBuffer[2] = 6;     // Polling Interval
  packetBuffer[3] = 0xEC;  // Peer Clock Precision
  // 8 bytes of zero for Root Delay & Root Dispersion
  packetBuffer[12]  = 49;
  packetBuffer[13]  = 0x4E;
  packetBuffer[14]  = 49;
  packetBuffer[15]  = 52;

  // all NTP fields have been given values, now
  // you can send a packet requesting a timestamp:
  Udp.beginPacket(address, 123); //NTP requests are to port 123
  Udp.write(packetBuffer, NTP_PACKET_SIZE);
  Udp.endPacket();
}
// End sendNTPpacket

// Begin GetTime
// GetTime breaks the UTC time up into an array for use.
void GetTime() {
  sendNTPpacket( timeServer );
  delay( 1000 );
  if ( Udp.parsePacket() ) {  
    Udp.read( packetBuffer, NTP_PACKET_SIZE );
    unsigned long highWord = word( packetBuffer[ 40 ], packetBuffer[ 41 ] );
    unsigned long lowWord = word( packetBuffer[ 42 ], packetBuffer[ 43 ] );  
    unsigned long secsSince1900 = highWord << 16 | lowWord;  
    unsigned long epoch = secsSince1900 - 2208988800UL;  
    UTCTime[ 0 ] = ( ( epoch % 86400L ) / 3600 );
    UTCTime[ 1 ] = ( ( epoch  % 3600 ) / 60 );
  }
}
// End GetTime

// Begin Webserver
// Webserver is meant to provide a simple interface to check the IP of your sensor and the current readings it has.
// It will refresh every 5 seconds unless stopped.
void Webserver() {
  EthernetClient Web = Server.available();
  int noChar = 0;
  if ( Web ) {
    boolean currentLineIsBlank = true;
    while ( Web.connected() ) {
      if ( Web.available() ) {
        char c = Web.read();
        noChar = 0;
        if (c == '\n' && currentLineIsBlank) {
          Web.println( "HTTP/1.1 200 OK" );
          Web.println( "Content-Type: text/html" );
          Web.println();
          Web.println( "<html>" );
          Web.println( "<head>" );
          Web.println( "<meta http-equiv='refresh' content='5'>" );
          Web.println( "<title>Sensor</title>" );
          Web.println( "</head>" );
          Web.println( "<body>" );
          Web.print( "Time: " );
          Web.print( UTCTime[ 0 ] );
          Web.print( ":" );
          if ( UTCTime[ 1 ] < 10 ) { // If the minute element is less than 10, add a 0 to the output
            Web.print( "0" );
          }
          Web.print( UTCTime[ 1 ] );
          Web.println( "UTC</br>" );
          for( int x = 0; x < WC.MaxSensors; x++ ){ // Run through the maximum number of sensors.
            if( WC.SensorReadings[ x ] != -1000 ) { // If the sensor value is not equal to -1000, display it
              Web.print( WC.SensorNames[ x ] );
              Web.print( "=" );
              Web.print( WC.SensorReadings[ x ] );
              Web.println( "</br>" );
            }
          }
          Web.println( "</body>" );
          Web.println( "</html>" );
          break;
        }
        if (c == '\n') {
          currentLineIsBlank = true;
        } 
        else if (c != '\r') {
          currentLineIsBlank = false;
        }
      }
      noChar++;
      if( noChar > 10000 ){
        if( Debug ){
          Serial.println( "Timed out." );
        }
        Web.stop();
      }
    }
    Web.stop();
  }
}
// End Webserver

// Begin Setup
void setup() {
  if( Debug ){ // If debugging is enabled
    Serial.begin( 115200 );
  }
	
  // Start Sensors
  dht.begin();
  // End Sensors
	
  // Start networking
  if( Networked ){ // If networking is enabled
    if ( Ethernet.begin( mac ) == 0 ) {
      Ethernet.begin( mac, ip );
    }
    delay( 1000 ); // Delay for Ethernet shield
    Server.begin();
    Udp.begin( LocalUDP ); // Start UDP for NTP
		
    // To be honest I am not sure why the next spots are necessary. If the ID and Key were added to the library
    // directly by someone. I could not get it to work nicely with a string or having them copied directly.
    for(int i = 0; i < 16; i++){ // Provide the ID information to the library.
      WC.ID[ i ] = WeatherID[ i ];
    }
    for(int i = 0; i < 32; i++){ // Provide the Key information to the library.
      WC.Key[ i ] = WeatherKey[ i ];
    }
    if( Debug ){ // If debugging is enabled
      Serial.print( "IP = " );
      Serial.println( Ethernet.localIP() );
    }
    WC.Init(); // Initialize the Weathercloud
  }
  // End Networking
}
// End Setup

// Begin Loop
// Main loop of the program
void loop() {
  if( millis() > NTPTime ) { // Is it time to perform an NTP request?
    if( Networked ){ // Is the device networked?
      GetTime(); // Check for the time.
    }
    NTPTime = millis() + NTPDelay; // Set the next time to perform an NTP check
  }
  if( millis() > SensorTime ) { // Is it time to check the sensors?
    // Content within here will need to be HEAVILY modified based on whatever sensors you are including.
    // In this case the program is only checking humidity and temperature.
    float h = dht.readHumidity();
    float t = dht.readTemperature();
    if( Debug ){ // Debugging purposes
      Serial.print( "Temp: " );
      Serial.print( t );
      Serial.print( "c / Humidity: ");
      Serial.print( h );
      Serial.print( "%" );
    }
    if( Networked ){ // Is the device networked?
      WC.SensorReadings[ 0 ] = int( ( t * 10 ) ); // Weathercloud expects an integer but the temperature has a decimal point
      WC.SensorReadings[ 10 ] = int( h );
    }
    SensorTime = millis() + SensorDelay; // Set the next time to perform a Sensor reading
  }
  if( Networked ){ // Is the device networked?
    WC.PostData(); // Post the data to Weathercloud. It makes sure it is no faster than their maximum of 10 minute postings.
    Webserver(); // Provide the webpage for checking sensor status
    Ethernet.maintain(); // Maintain the ethernet
  }
}
// End Loop